@@ -1,9 +1,10 @@
 import sys
 import fire
-from txt2img.core import core, save_images, show_images
+from txt2img.core import core, save_tensor_image, display_image,get_unique_file_path
 from PIL import TiffTags
 import sys
 
+
 def save(
     *args,
     batch_size: int = 1,
@@ -26,10 +27,12 @@ def save(
         prompt = " ".join(args)
     else:
         prompt = args[0]
-
+    
+    prompt= prompt.replace("  ",' ')
     tf_img = core(prompt, batch_size, guidance_scale, upsample_temp)
-    out_path = prompt.replace(" ", "_") + ".png"
-    save_images(tf_img, out_path)
+    name = prompt.replace(" ", "_").strip()
+    out_path = get_unique_file_path('.',name,'png')
+    save_tensor_image(tf_img, out_path)
 
 
 def show_me(
@@ -49,7 +52,10 @@ def show_me(
     """
 
     tf_img = core(prompt, batch_size, guidance_scale, upsample_temp)
-    show_images(tf_img)
+    name = prompt.replace(" ", "_").strip()
+    out_path = get_unique_file_path('.',name,'png')
+    save_tensor_image(tf_img, out_path)
+    display_image(out_path)
 
 # function alias
 show_a = show_me
