@@ -1,9 +1,18 @@
 import fire
 import torch as th
-from IPython.display import display
-from PIL import Image
+from IPython.display import display,Image
+
+#from PIL import Image
 import sys
 from importlib import reload  # Python 3.4+
+import time
+import os
+from numpy import random
+import os
+import IPython
+from matplotlib import pyplot as plt
+
+
 
 from glide_text2im.download import load_checkpoint
 from glide_text2im.model_creation import (
@@ -12,12 +21,26 @@ from glide_text2im.model_creation import (
     model_and_diffusion_defaults_upsampler,
 )
 
+
+from IPython.display import Image
+from IPython.core.display import HTML
+from IPython.core.display import display
+import numpy as np
+
+
+
 # This notebook supports both CPU and GPU.
 # On CPU, generating one sample may take on the order of 20 minutes.
 # On a GPU, it should be under a minute.
 has_cuda = th.cuda.is_available()
 device = th.device("cpu" if not has_cuda else "cuda")
+is_in_colab = 'google.colab' in sys.modules
 
+if not has_cuda:
+    print("Warning: No cuda gpu avalible:")
+    if is_in_colab:
+        print("Select a gpu runtime by clicking,")
+        print("\t 'Runtime' > 'Change runtime type' > 'Hardware accelerator' > 'GPU'")
 
 def _create_base_model(options):
     """
@@ -67,32 +90,6 @@ def _create_upsampler_model(options_up):
     return model_up, diffusion_up
 
 
-def show_images(batch: th.Tensor):
-    """Display a batch of images inline.
-
-    Args:
-      batch: th.Tensor:
-
-    """
-    scaled = ((batch + 1) * 127.5).round().clamp(0, 255).to(th.uint8).cpu()
-    reshaped = scaled.permute(2, 0, 3, 1).reshape([batch.shape[2], -1, 3])
-    display(Image.fromarray(reshaped.numpy()))
-
-
-def save_images(batch: th.Tensor, out_path):
-    """Display a batch of images inline.
-
-    Args:
-      batch: th.Tensor:
-      out_path:
-
-    Returns:
-
-    """
-    scaled = ((batch + 1) * 127.5).round().clamp(0, 255).to(th.uint8).cpu()
-    reshaped = scaled.permute(2, 0, 3, 1).reshape([batch.shape[2], -1, 3])
-    Image.fromarray(reshaped.numpy()).save(out_path)
-
 
 def generate_guidance_sampling_function(model, guidance_scale):
     """
@@ -257,14 +254,26 @@ def upsample_the_64x64_samples(
 
 
 
-def check_pillow_import():
+def pillow_is_updated():
     try:
+        import PIL
         from PIL.TiffTags import IFD
-    except :
-        warning = "An old version of Pillow is already imported. You need to restart the runtime.: \n"
+        assert PIL.__version__ == '9.0.0'
+        return True
+    except:
+        return False
+
+
+
+def check_pillow_import():
+    if not pillow_is_updated():
+        warning = "An old version of Pillow is already imported. You need to restart the runtime: \n"
         answer = input(f"{warning}. Restart runtime? [y/N]")
         if answer in ['y','Y','yes','Yes', 'YES'] or answer is None:
-            os.kill(os.getpid(), 9)
+            print("Restarting runtime. You will need to reimport any packages you were using. ")
+            for i in range(5):
+                os.kill(os.getpid(), 9)
+                time.sleep(1)
 
 
 def core(
@@ -284,11 +293,79 @@ def core(
     Returns:
 
     """
-    check_pillow_import()
+    #check_pillow_import()
     print(f"Generate image of '{prompt}':")
     samples = sample_from_base_model(prompt, batch_size, guidance_scale)
     up_samples = upsample_the_64x64_samples(samples, prompt, batch_size, upsample_temp)
     return up_samples
 
 
-check_pillow_import()
+
+
+
+def get_unique_file_path(out_dir,file_name,suffix):
+
+  file_path = f'{out_dir}/{file_name}.{suffix}'
+  if not os.path.isfile(file_path):
+    return file_path
+  i = 1
+  while os.path.isfile(f'{out_dir}/{file_name} [{i}].{suffix}'):
+    i+=1
+  return  f'{out_dir}/{file_name} [{i}].{suffix}'
+
+
+def save_numpy_image(data,out_path):
+    fig=plt.figure()
+    ax=fig.add_subplot(1,1,1)
+    plt.axis('off')
+    plt.tight_layout()
+    plt.subplots_adjust(left=0, right=0.5)
+    plt.imshow(data)
+    plt.savefig(out_path, bbox_inches='tight',pad_inches = 0)
+    plt.close()
+
+def save_tensor_image(batch: th.Tensor, out_path:str ):
+    scaled = ((batch + 1) * 127.5).round().clamp(0, 255).to(th.uint8).cpu()
+    reshaped = scaled.permute(2, 0, 3, 1).reshape([batch.shape[2], -1, 3])
+    img_array = reshaped.numpy()
+    save_numpy_image(img_array,out_path)
+
+def display_image(file_path):
+    display( Image(file_path))
+
+
+
+#check_pillow_import()
+
+
+
+
+'''
+def show_images(batch: th.Tensor):
+    """Display a batch of images inline.
+
+    Args:
+      batch: th.Tensor:
+
+    """
+    scaled = ((batch + 1) * 127.5).round().clamp(0, 255).to(th.uint8).cpu()
+    reshaped = scaled.permute(2, 0, 3, 1).reshape([batch.shape[2], -1, 3])
+    img_array = reshaped.numpy()
+    
+    display(Image.fromarray(img_array))
+
+
+def save_images(batch: th.Tensor, out_path):
+    """Display a batch of images inline.
+
+    Args:
+      batch: th.Tensor:
+      out_path:
+
+    Returns:
+
+    """
+    scaled = ((batch + 1) * 127.5).round().clamp(0, 255).to(th.uint8).cpu()
+    reshaped = scaled.permute(2, 0, 3, 1).reshape([batch.shape[2], -1, 3])
+    Image.fromarray(reshaped.numpy()).save(out_path)
+'''
